@@ -31,9 +31,7 @@ struct _CloudProvidersProvider
     GList *accounts;
     gchar *manager_bus_name;
     gchar *manager_object_path;
-    gchar *provider_object_path;
 
-    GHashTable* accounts_object_managers;
     GDBusConnection *bus;
     CloudProvidersDbusProvider *proxy;
     GCancellable *cancellable;
@@ -47,7 +45,7 @@ G_DEFINE_TYPE (CloudProvidersProvider, cloud_providers_provider, G_TYPE_OBJECT)
  * @short_description: Base object for representing a single provider for clients.
  * @include: src/cloudprovidersprovider.h
  *
- * #CloudProvidersProvider is the basic object object for client implementators
+ * #CloudProvidersProvider is the basic object object for client implementers
  * that contains the properties of the provider such as name, and the accounts
  * associated with it.
  */
@@ -71,6 +69,20 @@ static GParamSpec *properties [N_PROPS];
 
 static void
 on_name_changed (CloudProvidersProvider *self);
+static void
+on_cloud_providers_object_manager_object_added (GDBusObjectManager *manager,
+                                                GDBusObject        *object,
+                                                gpointer            user_data);
+
+static void
+on_cloud_providers_object_manager_object_removed (GDBusObjectManager *manager,
+                                                  GDBusObject        *object,
+                                                  gpointer            user_data);
+
+static void
+on_cloud_providers_object_manager_name_owner_changed (GObject    *object,
+                                                      GParamSpec *pspec,
+                                                      gpointer    user_data);
 
 CloudProvidersProvider *
 cloud_providers_provider_new (const gchar *bus_name,
@@ -85,23 +97,48 @@ cloud_providers_provider_new (const gchar *bus_name,
     return self;
 }
 
+static void
+cloud_providers_provider_dispose (GObject *object)
+{
+    CloudProvidersProvider *self = (CloudProvidersProvider *)object;
+
+    g_cancellable_cancel (self->cancellable);
+    g_clear_object (&self->cancellable);
+
+    g_list_free_full (self->accounts, g_object_unref);
+    self->accounts = NULL;
+
+    if (self->proxy)
+        g_signal_handlers_disconnect_by_func (self->proxy, G_CALLBACK (on_name_changed), self);
+    g_clear_object (&self->proxy);
+
+    if (self->manager)
+    {
+        g_signal_handlers_disconnect_by_func (self->manager,
+                                              G_CALLBACK (on_cloud_providers_object_manager_name_owner_changed),
+                                              self);
+        g_signal_handlers_disconnect_by_func (self->manager,
+                                              G_CALLBACK (on_cloud_providers_object_manager_object_added),
+                                              self);
+        g_signal_handlers_disconnect_by_func (self->manager,
+                                              G_CALLBACK (on_cloud_providers_object_manager_object_removed),
+                                              self);
+    }
+
+    g_clear_object (&self->manager);
+    g_clear_object (&self->bus);
+
+    G_OBJECT_CLASS (cloud_providers_provider_parent_class)->dispose (object);
+}
+
 static void
 cloud_providers_provider_finalize (GObject *object)
 {
     CloudProvidersProvider *self = (CloudProvidersProvider *)object;
 
-    g_list_free_full (self->accounts, g_object_unref);
-    g_free (self->name);
-    if (self->proxy != NULL)
-    {
-        g_signal_handlers_disconnect_by_data (self->proxy, self);
-    }
-    g_clear_object (&self->proxy);
-    if (self->manager)
-    {
-        g_signal_handlers_disconnect_by_data (self->manager, self);
-    }
-    g_clear_object (&self->manager);
+    g_clear_pointer (&self->name, g_free);
+    g_clear_pointer (&self->manager_bus_name, g_free);
+    g_clear_pointer (&self->manager_object_path, g_free);
 
     G_OBJECT_CLASS (cloud_providers_provider_parent_class)->finalize (object);
 }
@@ -143,7 +180,7 @@ cloud_providers_provider_set_property (GObject      *object,
         case PROP_NAME:
         {
             g_free (self->name);
-            self->name = g_strdup (g_value_get_string (value));
+            self->name = g_value_dup_string (value);
             g_object_notify_by_pspec (object, properties[PROP_NAME]);
         }
         break;
@@ -162,6 +199,7 @@ cloud_providers_provider_class_init (CloudProvidersProviderClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+    object_class->dispose = cloud_providers_provider_dispose;
     object_class->finalize = cloud_providers_provider_finalize;
     object_class->get_property = cloud_providers_provider_get_property;
     object_class->set_property = cloud_providers_provider_set_property;
@@ -172,13 +210,14 @@ cloud_providers_provider_class_init (CloudProvidersProviderClass *klass)
                              "Name of the provider",
                              NULL,
                              (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-    g_object_class_install_property (object_class, PROP_NAME,
-                                     properties [PROP_NAME]);
+    g_object_class_install_properties (object_class,
+                                       N_PROPS,
+                                       properties);
 
   /**
    * CloudProviderProvider::accounts-changed
    *
-   * This signal is emmited by a provider if the number of accounts changed.
+   * This signal is emitted by a provider if the number of accounts changed.
    */
   signals [ACCOUNTS_CHANGED] =
     g_signal_new ("accounts-changed",
@@ -193,7 +232,7 @@ cloud_providers_provider_class_init (CloudProvidersProviderClass *klass)
   /**
    * CloudProviderProvider::removed
    *
-   * This signal is emmited by a provider when the provider is removed in DBUS.
+   * This signal is emitted by a provider when the provider is removed in DBUS.
    */
   signals [REMOVED] =
     g_signal_new ("removed",
@@ -252,7 +291,7 @@ update_cloud_providers_accounts (CloudProvidersProvider *self)
         {
             CloudProvidersAccount *account;
 
-            account = cloud_providers_account_new (CLOUD_PROVIDERS_DBUS_ACCOUNT_PROXY (cloud_providers_dbus_object_peek_account (object)));
+            account = cloud_providers_account_new (G_DBUS_PROXY (cloud_providers_dbus_object_peek_account (object)));
 
             self->accounts = g_list_append (self->accounts, account);
             g_object_unref (interface);
@@ -291,7 +330,7 @@ on_cloud_providers_object_manager_object_removed (GDBusObjectManager *manager,
 static void
 on_cloud_providers_object_manager_name_owner_changed (GObject    *object,
                                                       GParamSpec *pspec,
-                                                      gpointer user_data)
+                                                      gpointer    user_data)
 {
     CloudProvidersProvider *self;
 
@@ -318,15 +357,13 @@ on_object_manager_created (GObject      *source_object,
                            GAsyncResult *res,
                            gpointer      user_data)
 {
-    CloudProvidersProvider *self;
-    GError *error = NULL;
+    CloudProvidersProvider *self = CLOUD_PROVIDERS_PROVIDER (user_data);
+    g_autoptr(GError) error = NULL;
 
-    self = CLOUD_PROVIDERS_PROVIDER (user_data);
-    self->manager = cloud_providers_dbus_object_manager_client_new_for_bus_finish (res, &error);
+    self->manager = cloud_providers_dbus_object_manager_client_new_finish (res, &error);
     if (error != NULL)
     {
         g_printerr ("Error getting object manager client: %s", error->message);
-        g_error_free (error);
         return;
     }
 
@@ -348,36 +385,31 @@ on_bus_acquired (GObject      *source_object,
                  GAsyncResult *res,
                  gpointer      user_data)
 {
-  GError *error = NULL;
-  CloudProvidersProvider *self;
-  GDBusConnection *bus;
+  CloudProvidersProvider *self = CLOUD_PROVIDERS_PROVIDER (user_data);
+  g_autoptr(GError) error = NULL;
 
-  bus = g_bus_get_finish (res, &error);
+  self->bus = g_bus_get_finish (res, &error);
   if (error != NULL)
     {
-      if (error->code != G_IO_ERROR_CANCELLED)
-        g_warning ("Error acdquiring bus for cloud provider %s", error->message);
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+        g_debug ("Error acquiring bus for cloud provider: %s", error->message);
       return;
     }
 
-  self = CLOUD_PROVIDERS_PROVIDER (user_data);
-  self->bus = bus;
-  g_clear_object (&self->cancellable);
-  self->cancellable = g_cancellable_new ();
-
-  cloud_providers_dbus_object_manager_client_new_for_bus (G_BUS_TYPE_SESSION,
-                                                          G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_NONE,
-                                                          self->manager_bus_name,
-                                                          self->manager_object_path,
-                                                          NULL,
-                                                          on_object_manager_created,
-                                                          self);
+  cloud_providers_dbus_object_manager_client_new (self->bus,
+                                                  G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_NONE,
+                                                  self->manager_bus_name,
+                                                  self->manager_object_path,
+                                                  self->cancellable,
+                                                  on_object_manager_created,
+                                                  self);
 }
 
 
 static void
 cloud_providers_provider_init (CloudProvidersProvider *self)
 {
+    self->cancellable = g_cancellable_new ();
     g_bus_get (G_BUS_TYPE_SESSION,
                self->cancellable,
                on_bus_acquired,
